/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.elmo.impl;

import it.tidalwave.util.Finder.SortDirection;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import it.tidalwave.openrdf.elmo.ElmoFinder;
import it.tidalwave.bluebill.taxonomy.Finder;
import it.tidalwave.bluebill.taxonomy.Taxon;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class ElmoTaxonomyFinder<T> extends ElmoFinder<T> implements Finder<T>
  {
    private final static long serialVersionUID = 23435256347568673L;
    
    private static final Logger log = LoggerFactory.getLogger(ElmoTaxonomyFinder.class);

    private static final Map<Class<?>, Class<?>> IMPLEMENTATION_MAP = new HashMap<Class<?>, Class<?>>();

    protected String name;

    static
      {
        IMPLEMENTATION_MAP.put(Taxon.class, ElmoTaxon.class);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected ElmoTaxonomyFinder (final @Nonnull Class<T> entityClass)
      {
        super(entityClass,
              (Class<T>)IMPLEMENTATION_MAP.get(entityClass),
              "PREFIX skos: <http://www.w3.org/2004/02/skos/core#>\n");
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    @Override
    public Finder<T> name (final @Nonnull String name)
      {
        this.name = name;
        return this;
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected void process (@Nonnull Set<String> auxStatements)
      {
        if (name != null)
          {
            auxStatements.add(String.format("?%s rdfs:label '%s'", entityName, name));
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    protected it.tidalwave.util.Finder<T> createClone()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    // FIXME: the following methods are needed because ElmoFinder doesn't support ExtendedFinder
    @Override
    public Finder<T> from (int firstResult) 
      {
        return (Finder<T>)super.from(firstResult);
      }

    @Override
    public Finder<T> max (int maxResults)   
      {
        return (Finder<T>)super.max(firstResult);
      }

    @Override
    public Finder<T> sort (it.tidalwave.util.Finder.SortCriterion criterion) 
      {
        return (Finder<T>)super.sort(criterion);
      }

    @Override
    public Finder<T> sort (it.tidalwave.util.Finder.SortCriterion criterion, SortDirection direction)
      {
        return (Finder<T>)super.sort(criterion, direction);
      }
  }
